package chess.graphics;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.event.MouseInputListener;

import chess.board.Board;
import chess.board.Move;
import chess.board.pieces.Piece;

/**Gui Board class, visual representation of the board
 * @author Garon
 *
 */
public class GuiBoard extends JFrame implements MouseInputListener {

	private static final long serialVersionUID = 4452528495794828156L;
	
	private Board myBoard;

	private Color[][] highLites;		//array of highlighted squares
	
	private boolean firstClick = true;	//checks whether first click
	private Point firstPixelPoint;		//first Point clicked
	private Point secondPixelPoint;		//second point clicked
	public Move currentMove;			//current move to be made
	
	public boolean moveSet = false;		//checks whether move has been made
	public boolean yourTurn = false;	//checks if its players turn
	public boolean inCheck = false;		//true if player is in check

	/**Constructor for GuiBoard
	 * @param title		Title of the window
	 */
	public GuiBoard(String title) {
		super(title);
		myBoard = new Board();
		highLites = new Color[8][8];
	    createUI();
	}
	
	/**Creates the UI for the game
	 * 
	 */
	protected void createUI() {
		setSize(550, 500);
		center();

		addMouseListener(this);
		addMouseMotionListener(this);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
	}
	
	/**Centers the Gui on the screen
	 * 
	 */
	public void center() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getSize();
		int x = (screenSize.width - frameSize.width) / 2;
		int y = (screenSize.height - frameSize.height) / 2;
		setLocation(x, y);
	}
	
	/* (non-Javadoc)
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		this.drawBoard(g);
	}

	/**Paints the board into the window
	 * @param g		graphics to be displayed
	 */
	public void drawBoard(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;

		// set the background
		g2.setPaint(Color.DARK_GRAY);
		g2.fill3DRect(0, 0, 550, 500, true);
		
		boolean black = true;
		
		//Draws the board and places the pieces
		int h = 0;
		for (int i = 400; i >= 50; i -= 50) {
			for (int j = 50; j < 450; j += 50) {
				Point location = new Point(j,i);
				//Highlights the area clicked
				if(highLites[convertToIndex(location).x][convertToIndex(location).y] != null){
					g2.setPaint(highLites[convertToIndex(location).x][convertToIndex(location).y]);
				}
				else{ 
					if (black) {
						g2.setPaint(Color.GRAY);
					} else {
						g2.setPaint(Color.LIGHT_GRAY);
					}
				}
				g2.fill3DRect(j, i, 50, 50, true);
				Piece p = myBoard.getBoard()[(j / 50) - 1][h];
				if (p != null) {
					g2.drawImage(p.getImage(), j, i, null);
				}
				black = !black;
			}
			black = !black;
			h++;
		}
		
		//if its your turn it prints it to the screen to tell you
		if(yourTurn){
			Font f = new Font("Arial", Font.BOLD, 18);
			g2.setFont(f);
			g2.setPaint(Color.green);
			g2.drawString("Your Turn", 450, 150);			
		}		
		if(inCheck){
			Font f = new Font("Arial", Font.BOLD, 18);
			g2.setFont(f);
			g2.setPaint(Color.green);
			g2.drawString("Check", 450, 200);
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent arg0) {
		if(yourTurn){
			if(firstClick){		//if its the first click it saves the point
				firstPixelPoint = arg0.getPoint();
				Point firstIndexPoint = new Point();
				firstIndexPoint = convertToIndex(firstPixelPoint);
				colorSquare(Color.red, firstIndexPoint);
				repaint();
				firstClick = false;
			}
			else{				//if its the second click it saves the point
				secondPixelPoint = arg0.getPoint();
				Point firstIndexPoint = new Point();
				Point secondIndexPoint = new Point();
				firstIndexPoint = convertToIndex(firstPixelPoint);	//converts both points
				secondIndexPoint = convertToIndex(secondPixelPoint);					
				firstClick = true;
				for(int i = 0; i < 8; i++){			//resets the highlites array
					for(int j = 0; j < 8; j++){
						highLites[j][i] = null;
					}
				}
				currentMove = new Move(firstIndexPoint, secondIndexPoint);	//saves converted points to currentMove
				repaint();
				moveSet = true;			//sets that move has been done
			}
		}
		//System.out.println(firstPixelPoint);
	}
	
	/**Sets the area in the highlight area to a given color
	 * @param c	Colot to set the positio to
	 * @param p	Position to be set
	 */
	public void colorSquare(Color c, Point p){
		highLites[p.x][p.y] = c;		
	}
	
	/**Converts a pixel point into an index location on the piece array
	 * @param P		Point to be converted
	 * @return		Converted Point
	 */
	public Point convertToIndex(Point P){
		Point indexP =  new Point();
		indexP.x = (P.x)/50 - 1;
		indexP.y = 8 - (int)((P.y)/50);
		return indexP;
	}
	
	//None of the following mouse actions have been implemented

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public Move getCurrentMove() {
		firstClick = true;	//resets the click
		return currentMove;
	}

	/**Gets the board of the GUI
	 * @return Returns the myBoard.
	 */
	public Board getMyBoard() {
		return myBoard;
	}

	/**Sets the board of the GUI
	 * @param myBoard The myBoard to set.
	 */
	public void setMyBoard(Board myBoard) {
		this.myBoard = myBoard;
	}

}
