package chess.player;

import java.util.ArrayList;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;
import chess.player.heuristics.Aggressive;
import chess.player.heuristics.Defensive;
import chess.player.heuristics.Heuristic;
import chess.player.heuristics.Neutral;

/**AI Player, extends Player class
 * @author Garon
 *
 */
public class AiPlayer extends Player {
	private Board myBoard;	//This player's copy of the board
	private int depthSet;
	private int alpha;
	private int beta;
	private Heuristic playStyle;
	

	/**Constructor for AI Player
	 * @param owner	Player's color
	 */
	public AiPlayer(PlayerColor owner, PlayDifficulty diff, PlayStyle style) {
		super(owner);
		myBoard = new Board();	
		if(diff == PlayDifficulty.EASY){
			depthSet = 2;
		}
		else if(diff == PlayDifficulty.MEDIUM){
			depthSet = 4;
		}
		else if(diff == PlayDifficulty.HARD){
			depthSet = 5;
		}
		if(style == PlayStyle.NEUTRAL){
			playStyle = new Neutral();
		}
		else if(style == PlayStyle.DEFENSIVE){
			playStyle = new Defensive();
		}
		else if(style == PlayStyle.AGGRESSIVE){
			playStyle = new Aggressive();
		}	
		if(owner == PlayerColor.WHITE){
			alpha = Integer.MIN_VALUE;
			beta = Integer.MAX_VALUE;
		}
		else{
			alpha = Integer.MAX_VALUE;
			beta = Integer.MIN_VALUE;
		}
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#startGame()
	 */
	@Override
	public void startGame(PlayerColor owner) {
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#getMove()
	 */
	@Override
	public Move getMove() {
		ArrayList<Move> bestMoves = new ArrayList<Move>();
		int bestValue;
		if(this.owner == PlayerColor.WHITE) {
			bestValue = Integer.MIN_VALUE;
		} else {
			bestValue = Integer.MAX_VALUE;
		}
		Move[] allMoves = myBoard.getAllLegalMoves(this.owner);		
		
		for(int i = allMoves.length-1; i >= 0; i--){
			Board temp = new Board(myBoard);
			temp.movePiece(this.owner, allMoves[i].startPoint, allMoves[i].endPoint);
			int value = alphaBeta(depthSet, -beta, -alpha, temp, Board.switchColor(this.owner));
			if(this.owner == PlayerColor.WHITE) {
				if(value == bestValue){
					bestMoves.add(allMoves[i]);
					bestValue = value;
				} 
				else if(value > bestValue) {
					bestMoves.clear();
					bestMoves.add(allMoves[i]);
					bestValue = value;
				}
			}
			else {
				if(value == bestValue){
					bestMoves.add(allMoves[i]);
					bestValue = value;
				} 
				else if(value < bestValue) {
					bestMoves.clear();
					bestMoves.add(allMoves[i]);
					bestValue = value;
				}
			}
			
		}
		
		//pick random best move
		int index = (int) Math.round(Math.random() * (bestMoves.size()-1));
		Move bestMove = bestMoves.get(index);				
		myBoard.movePiece(this.owner, bestMove.startPoint, bestMove.endPoint);
		return bestMove;
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#sendMove(chess.board.Move)
	 */
	@Override
	public void sendMove(Move m) {
		myBoard.movePiece(Board.switchColor(getOwner()),m.startPoint, m.endPoint);

	}	
	
	
	/**Alpha Beta Pruning Algorithm
	 * @param depth		Depth to be searched
	 * @param alpha		Alpha value
	 * @param beta		Beta Value
	 * @param b			Board to be searched
	 * @param c			Current Players Color
	 * @return			Value of Best move
	 */
	public int alphaBeta(int depth, int alpha, int beta, Board b, PlayerColor c){
		if(depth == 0) {
			return playStyle.evaluateBoardState(c, b);
		}
		Move[] allMoves = b.getAllLegalMoves(c);
		for(int i = allMoves.length-1; i >= 0; i--){
			Board temp = new Board(b);
			temp.movePiece(c, allMoves[i].startPoint, allMoves[i].endPoint);
			int value = -1*alphaBeta(depth-1, -beta, -alpha, temp, Board.switchColor(c));
			if(value > alpha){
				alpha = value;
			}
			if(value >= beta){
				return value;
			}			
		}		
		return alpha;
	}

	@Override
	public void gameTerminated(boolean isWinner) {		
	}
}
